﻿using AutoMapper;
using Common;
using Quartz;
using Services.Interface;
using JmpDehumidifierLib;
using Models.Table;
using static JmpDehumidifierLib.Dehumidifier;

namespace APIs.TimedTasks
{
    public class DehumidifierTask : IHostedService
    {
        //private readonly ILogger<DehumidifierTask> _logger;
        public IMapper Mapper { get; }

        private readonly IDevHistoryService _devHistoryService;
        private readonly IDevService _devService;

        public DehumidifierTask(IMapper mapper, IDevHistoryService devHistoryService, IDevService devService/*ILogger<DehumidifierTask> logger*/)
        {
            _devHistoryService = devHistoryService;
            _devService = devService;
            Mapper = mapper;
            //_logger = logger;
        }

        Dictionary<string, Dehumidifier> devList = new Dictionary<string, Dehumidifier>();

        public Task StartAsync(CancellationToken cancellationToken)
        {
            //_logger.LogInformation("订阅事件开始");

            //开启本组织机构下所有除湿机并订阅温湿度变化事件
            try
            {
                List<Dev>? devs = _devService.Query().Result;

                if (devs != null)
                {
                    devList = devs.Select(s => new { Key = s.devIp, Value = new Dehumidifier(s.devIp, Convert.ToInt32(s.devPort)) })
                        .ToDictionary(s => s.Key, s => s.Value);

                    var openIpList = devs.Where(s => s.setState == 1).Select(s => s.devIp).ToList();

                    foreach (var item in devList)
                    {
                        if (openIpList.Contains(item.Key))
                        {
                            if (item.Value.Open())
                            {
                                try//避免影响设备温湿度获取
                                {
                                    var mySetSd = devs.FirstOrDefault(s => s.devIp.Equals(item.Key))?.setSd;
                                    if (!string.IsNullOrEmpty(mySetSd))
                                    {
                                        item.Value.SetHumid((byte)Convert.ToInt32(mySetSd));
                                    }
                                }catch (Exception ex) { }

                                item.Value.OnStatusChanged += new Dehumidifier.StatusChanged(OnStatusChanged);
                            }
                        }
                        else
                        {
                            item.Value.Close();
                        }
                    }
                }

            }
            catch (Exception ex)
            {
                //异常处理
            }
            return Task.CompletedTask;
        }

        private Dictionary<string, DevHistory> DevHistoryDic = new Dictionary<string, DevHistory>();
        private async void OnStatusChanged(MachineStatus args)
        {
            if (args.ErrorCode == 0)
            {
                var devIp = args.ip;
                var warehouseWd = args.Temp.ToString();  // 室内温度
                var warehouseSd = args.Humid.ToString(); // 室内湿度
                var devState = args.IsWorking ? 1 : 0;
                var devPort = args.port.ToString();

                var myDev = (await _devService.Query()).FirstOrDefault(s => s.devIp.Equals(devIp));
                if (myDev != null)
                {
                    DevHistory record = new DevHistory()
                    {
                        warehouseId = myDev.warehouseId,
                        devCode = myDev.devCode,
                        devIp = devIp,
                        devPort = devPort,
                        devName = myDev.devName,
                        devId = myDev.id,
                        devState = devState,
                        warehouseSd = warehouseSd,
                        warehouseWd = warehouseWd,
                        setSd = myDev.setSd,
                        createTime = DateTime.Now,
                    };

                    if (DevHistoryDic.TryGetValue(devIp, out DevHistory thisHistory))
                    {
                        if(thisHistory.warehouseWd.Equals(record.warehouseWd) 
                            && thisHistory.warehouseSd.Equals(record.warehouseSd)
                            && thisHistory.devState == record.devState)
                        {
                            return;
                        }
                        else
                        {
                            DevHistoryDic[devIp] = record;
                        }
                    }
                    else
                    {
                        DevHistoryDic.Add(record.devIp, record);
                    }

                    await _devHistoryService.Add(record);

                    myDev.updateTime = DateTime.Now;
                    myDev.devState = devState;
                    myDev.warehouseSd = warehouseSd;
                    myDev.warehouseWd = warehouseWd;

                    await _devService.Update(myDev);
                }

                Thread.Sleep(10000);
            }
            else
            {
                foreach (var item in devList)
                {
                    if (item.Key.Equals(args.ip))
                    {
                        Dehumidifier dev = new Dehumidifier(args.ip, args.port);
                        devList.Remove(item.Key);
                        devList.Add(args.ip, dev);
                        if (dev.Open())
                        {
                            dev.OnStatusChanged += new Dehumidifier.StatusChanged(OnStatusChanged);
                        }
                    }
                }
            }
        }

        public Task StopAsync(CancellationToken cancellationToken)
        {
            //_logger.LogInformation("订阅事件结束");
            // TODO: 在这里取消订阅事件
            return Task.CompletedTask;
        }
    }
}
