﻿using AutoMapper;
using JmpDehumidifierLib;
using LitJson;
using Models.Table;
using Services;
using Services.Interface;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Management;
using System.Net;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Common
{
    public class DehumidifierHelper
    {
        public IMapper Mapper { get; }

        private readonly IDevHistoryService _devHistoryService;
        private readonly IDevService _devService;

        public DehumidifierHelper(IMapper mapper,IDevHistoryService devHistoryService, IDevService devService)
        {
            _devHistoryService = devHistoryService;
            _devService = devService;
            Mapper = mapper;
        }

        Dictionary<string, Dehumidifier> devList = new Dictionary<string, Dehumidifier>();

        public async void OnStart()
        {
            try
            {
                List<Dev>? devs = await _devService.Query();

                if(devs != null)
                {
                    devList = devs.Select(s => new { Key = s.devIp, Value = new Dehumidifier(s.devIp, Convert.ToInt32(s.devPort)) })
                        .ToDictionary(s => s.Key, s => s.Value);

                    var openIpList = devs.Where(s => s.devState == 1).Select(s => s.devIp).ToList();

                    foreach (var item in devList)
                    {
                        if (openIpList.Contains(item.Key))
                        {
                            if (item.Value.Open())
                            {
                                item.Value.OnStatusChanged += new Dehumidifier.StatusChanged(OnStatusChanged);
                            }
                        }
                        else
                        {
                            item.Value.Close();
                        }
                    }
                }

            }
            catch (Exception ex)
            {
                //异常处理
            }
        }

        //控制设备
        private void ContorlDev(int isOpen,int Humid, string devIp = "")
        {
            try
            {
                if (string.IsNullOrEmpty(devIp))
                {
                    foreach (var item in devList)
                    {
                        item.Value.SetHumid((byte)Humid);//设定湿度

                        if (isOpen == 1)
                        {
                            item.Value.OpenMachine(true);
                        }

                        if (isOpen == 0)
                        {
                            item.Value.OpenMachine(false);
                        }
                    }
                }
                else
                {
                    if (devList.TryGetValue(devIp, out Dehumidifier dehumidifier))
                    {
                        dehumidifier.SetHumid((byte)Humid);//设定湿度

                        if (isOpen == 1)
                        {
                            dehumidifier.OpenMachine(true);
                        }

                        if (isOpen == 0)
                        {
                            dehumidifier.OpenMachine(false);
                        }
                    }
                }
            }
            catch (Exception ex)
            {

            }
        }


        private async void OnStatusChanged(MachineStatus args)
        {
            if (args.ErrorCode == 0)
            {
                var devIp = args.ip;
                var warehouseWd = args.Temp.ToString();  // 室内温度
                var warehouseSd = args.Humid.ToString(); // 室内湿度
                var devState = args.IsWorking ? 1 : 0;
                var devPort = args.port.ToString();

                //string data = "{'devCode':'" + devCode + "','devIp':'" + devIp + "','warehouseWd':'" + warehouseWd + "','warehouseSd':'" + warehouseSd + "','devState':'" + devState + "'}";
                //保存数据
                var myDev = (await _devService.Query()).FirstOrDefault(s => s.devIp.Equals(devIp));
                if(myDev != null)
                {
                    DevHistory record = new DevHistory()
                    {
                        warehouseId = myDev.warehouseId,
                        devCode = myDev.devCode,
                        devIp = devIp,
                        devPort = devPort,
                        devName = myDev.devName,
                        devId = myDev.id,
                        devState = devState,
                        warehouseSd = warehouseSd,
                        warehouseWd = warehouseWd,
                        setSd = myDev.setSd,
                        createTime = DateTime.Now,
                    };

                    await _devHistoryService.Add(record);
                }

            }
            else
            {
                foreach (var item in devList)
                {
                    if (item.Key.Equals(args.ip))
                    {
                        Dehumidifier dev = new Dehumidifier(args.ip, args.port);
                        devList.Remove(item.Key);
                        devList.Add(args.ip, dev);
                        if (dev.Open())
                        {
                            dev.OnStatusChanged += new Dehumidifier.StatusChanged(OnStatusChanged);
                        }
                    }
                }
            }

        }

        public static string GetMacAddress()
        {
            try
            {
                string mac = "";
                ManagementClass mc = new ManagementClass("Win32_NetworkAdapterConfiguration");
                ManagementObjectCollection moc = mc.GetInstances();
                foreach (ManagementObject mo in moc)
                {
                    if ((bool)mo["IPEnabled"] == true)
                    {
                        mac = mo["MacAddress"].ToString();
                        break;
                    }
                }
                return mac;
            }
            catch
            {
                return "unknow";
            }
        }


        public static string GetHttpRqst(string strUrl, string json = "")
        {
            try
            {

                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(strUrl);
                request.Method = "POST";

                if (!string.IsNullOrEmpty(json))
                {
                    request.ContentType = "application/json";
                    byte[] data = Encoding.UTF8.GetBytes(json);
                    request.ContentLength = data.Length;
                    using (Stream reqStream = request.GetRequestStream())
                    {
                        reqStream.Write(data, 0, data.Length);
                        reqStream.Close();
                    }
                }

                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream responseStream = response.GetResponseStream();
                StreamReader streamReader = new StreamReader(responseStream, Encoding.UTF8);
                string retString = streamReader.ReadToEnd();
                streamReader.Close();
                responseStream.Close();
                response.Close();
                return retString;

            }
            catch (Exception ex)
            {
                return string.Empty;
            }
        }

        public static string getKey(JsonData data, string key)
        {
            if (JsonDataContainsKey(data, key))
            {
                if (data[key] == null)
                {
                    return "";
                }
                else
                {
                    return data[key].ToString();
                }
            }
            else
            {
                return "";
            }
        }

        public static bool JsonDataContainsKey(JsonData data, string key)
        {
            bool result = false;
            if (data == null)
                return result;
            if (!data.IsObject)
            {
                return result;
            }
            IDictionary tdictionary = data as IDictionary;
            if (tdictionary == null)
                return result;
            if (tdictionary.Contains(key))
            {
                result = true;
            }
            return result;
        }
    }
}
