﻿using AutoMapper;
using Models.SqlModel;
using Models.Table;
using Repositories.IRepository.IBussiness;
using Repositories.IRepository.IUnitOfWork;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Repositories.Repository.Bussiness
{
    public class LogSummaryRepository : BaseRepository<LogSummary>, ILogSummaryRepository
    {
        protected readonly ILocalSugarUnitOfWork _suger;
        public IMapper mapper;
        public LogSummaryRepository(ILocalSugarUnitOfWork sugarUnitOfWork) : base(sugarUnitOfWork)
        {
            _suger = sugarUnitOfWork;
        }

        public async Task<bool> AddChannelLog(OrderMain? myOrder, LogSummary summary, List<Inventory> newInvs, Tuple<string,string> inv_states)
        {
            using (var context = _suger.GetDbClient())
            {
                try
                {
                    context.BeginTran();

                    if(newInvs != null && newInvs.Count() > 0)
                    {
                        var inv_rs = context.Insertable(newInvs).ExecuteCommand();

                        //采购单据的出入库记录加入库存id
                        summary.DetailList.ForEach(s =>
                        {
                            s.inventoryId = context.Queryable<Inventory>().First(n => n.epc.Equals(s.epc))?.id;
                        });
                    }

                    var logrs = context.InsertNav(summary).Include(z1 => z1.DetailList).ExecuteCommand();

                    //更新库存状态
                    var invIds = summary.DetailList.Select(a => a.inventoryId).ToList();
                    context.Updateable<Inventory>().Where(s => invIds.Contains(s.id))
                        .SetColumns(i => new Inventory() 
                        {  
                            state = newInvs != null && newInvs.Count() > 0 ? "2" : summary.outInState.Equals("out") ? "1" : "0", 
                            bussinessState = inv_states.Item1, 
                            equState = inv_states.Item2, 
                            updateTime = DateTime.Now 
                        }).ExecuteCommand();

                    //更新单据
                    if (myOrder != null)
                    {
                        //子表
                        var ids = context.Queryable<LogSummary>()
                            .Where(s => s.orderId.Equals(myOrder.id))
                            .Select(s => s.id).ToList();

                        List<LogDetail> details = context.Queryable<LogDetail>()
                            .Where(s => ids.Contains(s.summaryId)).ToList();

                        var detailSumDic = details
                            .GroupBy(s => new { s.equipmentSizeId, s.epc })//避免1个EPC重复出入库导致单据反复计数
                            .Select(s => new { size = s.Key.equipmentSizeId, epc = s.Key.epc })
                            .GroupBy(s => s.size)
                            .Select(s => new { size = s.Key, count = s.Count() })
                            .ToDictionary(s => s.size, s => s.count);

                        foreach (var item in myOrder.DetailList)
                        {
                            if (detailSumDic.ContainsKey(item.equipmentSize))
                            {
                                item.actualNum = detailSumDic[item.equipmentSize];
                            }
                        }

                        //主表
                        decimal sumPrice = Convert.ToDecimal(myOrder.DetailList.Select(s => s.actualNum * s.price).Sum());

                        myOrder.actualQuantity = summary.number;
                        myOrder.price = sumPrice;
                        myOrder.isUpload = 1;
                        myOrder.orderState = 0;
                        myOrder.updateTime = DateTime.Now;

                        var orderrs = context.UpdateNav(myOrder).Include(z1 => z1.DetailList).ExecuteCommand();

                        //调拨出库删除装备
                        if (inv_states.Item1.Equals("transfer"))
                        {
                            context.Deleteable<Inventory>().Where(s => invIds.Contains(s.id)).ExecuteCommand();
                        }
                    }

                    context.CommitTran();
                }
                catch (Exception e)
                {
                    return false;
                }
            }
            return true;
        }

        public async Task<bool> AddHandLogs(LogSummary model, List<Inventory> inv)
        {
            using (var context = _suger.GetDbClient())
            {
                try
                {
                    context.BeginTran();

                    var result = context.Updateable(inv).WhereColumns(it => new { it.epc }).UpdateColumns(it => new { it.state, it.updateTime }).ExecuteCommand();//实体有多少列更新多少列
                    var logSum = context.InsertNav(model)
                        .Include(z1 => z1.DetailList)
                        .ExecuteCommand();

                    context.CommitTran();

                }
                catch (Exception e)
                {
                    throw e;
                }
            }
            return true;
        }

        public async Task<bool> AddLogs(LogSummary model, List<Inventory> inv, List<Car> cars)
        {
            using (var context = _suger.GetDbClient())
            {
                try
                {
                    //var invs = await context.Queryable<Inventory>().Where(x =>
                    //model.DetailList.Select(x => x.epc).Contains(x.epc)).ToListAsync();

                    var cabinetInv = inv.Where(s => s.state == "0").ToList();

                    for(int i = 0; i < cabinetInv.Count(); i++)
                    {
                        cabinetInv[i].locationType = "1";
                    }

                    context.BeginTran();

                    context.Updateable(cars).WhereColumns(it => new { it.id }).UpdateColumns(it => new { it.state, it.updateTime }).ExecuteCommand();//实体有多少列更新多少列

                    var result = context.Updateable(inv).WhereColumns(it => new { it.epc }).UpdateColumns(it => new { it.state, it.updateTime }).ExecuteCommand();//实体有多少列更新多少列

                    var result1 = context.Updateable(cabinetInv).WhereColumns(it => new { it.epc }).UpdateColumns(it => new { it.locationType }).ExecuteCommand();

                    var logSum = context.InsertNav(model)
                        .Include(z1 => z1.DetailList)
                        .ExecuteCommand();

                    context.CommitTran();

                }
                catch (Exception e)
                {
                    throw e;
                }
            }
            return true;
        }
    }
}
